<?php

namespace Drupal\spa\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\spa\SpaInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for SPA routes.
 */
class SpaController extends ControllerBase {

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The bare HTML page renderer.
   *
   * @var \Drupal\Core\Render\BareHtmlPageRendererInterface
   */
  protected $bareHtmlPageRenderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->routeMatch = $container->get('current_route_match');
    $instance->bareHtmlPageRenderer = $container->get('bare_html_page_renderer');
    return $instance;
  }

  /**
   * Builds the iframe response.
   */
  public function iframe() {
    $argument_values = $this->routeMatch->getParameters()->all();
    /** @var SpaInterface $spa */
    $spa = $this->routeMatch->getRouteObject()->getOption('spa');

    $build['content'] = [
      '#type' => 'inline_template',
      '#template' => $spa->getTemplate(),
      '#context' => $argument_values,
      '#attached' => [
        'library' => [
          'spa/' . $spa->libraryName(),
          'spa/iframe-resizer.inside'
        ],
        'drupalSettings' => $argument_values,
      ],
      '#cache' => ['tags' => $spa->getCacheTags()],
    ];

    return $this->bareHtmlPageRenderer->renderBarePage($build, 'SPA iframe', 'iframe_page', []);
  }

  /**
   * Builds the library response.
   */
  public function library(SpaInterface $spa, $library_uuid) {
    if ($library = $spa->getLibrary($library_uuid)) {
      return $library->getContent();
    }
  }

}
