<?php

namespace Drupal\spa\Event;

use Drupal\Component\EventDispatcher\Event;
use Drupal\spa\SpaArgumentInterface;
use Drupal\spa\SpaInterface;

class PassSpaArgumentEvent extends Event {

  /**
   * The SPA config entity.
   *
   * @var SpaInterface
   */
  protected $spa;

  /**
   * The SPA argument plugin instance.
   *
   * @var SpaArgumentInterface
   */
  protected $argument;

  /**
   * The argument value.
   *
   * @var mixed
   */
  protected $value;

  /**
   * PassSpaArgumentEvent constructor.
   *
   * @param SpaInterface $spa
   *   The SPA config entity.
   * @param SpaArgumentInterface $argument
   *   The SPA argument plugin instance.
   */
  public function __construct(SpaInterface $spa, SpaArgumentInterface $argument) {
    $this->spa = $spa;
    $this->argument = $argument;
    $this->value = $argument->getArgumentValue();
  }

  /**
   * @return SpaInterface
   */
  public function getSpa(): SpaInterface {
    return $this->spa;
  }

  /**
   * @return SpaArgumentInterface
   */
  public function getArgument(): SpaArgumentInterface {
    return $this->argument;
  }

  /**
   * @return mixed
   */
  public function getValue() {
    return $this->value;
  }

  /**
   * @param mixed $value
   */
  public function setValue($value): void {
    $this->value = $value;
  }

}
