<?php

namespace Drupal\spa\EventSubscriber;

use Drupal\Core\Entity\EntityInterface;
use Drupal\spa\Event\PassSpaArgumentEvent;
use Drupal\spa\Plugin\SpaArgument\ContextSpaArgument;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * SPA SpaArgumentEntityContextSubscriber event subscriber.
 */
class SpaArgumentEntityContextSubscriber implements EventSubscriberInterface {

  /**
   * Spa argument event handler.
   *
   * @param \Drupal\spa\Event\PassSpaArgumentEvent $event
   *   Response event.
   */
  public function downcastArguments(PassSpaArgumentEvent $event) {
    $argument = $event->getArgument();
    $argument_value = $event->getValue();
    // Downcast entity context values.
    if ($event->getSpa()->renderAsIframe() && $argument instanceof ContextSpaArgument && $argument_value instanceof EntityInterface) {
      if (strpos($argument->getDataType(), 'entity:') === 0 ) {
        $event->setValue($argument_value->id());
      }
    }
  }
  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      'spa.pass_argument' => ['downcastArguments'],
    ];
  }

}
