<?php

namespace Drupal\spa\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\spa\SpaInterface;

/**
 * Spa form.
 *
 * @property \Drupal\spa\SpaInterface $entity
 */
class SpaForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#description' => $this->t('Label for the spa.'),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\spa\Entity\Spa::load',
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $form['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $this->entity->status(),
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $this->entity->get('description'),
      '#description' => $this->t('Description of the SPA.'),
    ];

    $form['template'] = [
      '#type' => 'textarea',
      '#title' => $this->t('TWIG Template'),
      '#default_value' => $this->entity->get('template'),
      '#description' => $this->t('The SPAs TWIG template.'),
    ];

    $form['iframe'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Render inside an iframe element'),
      '#default_value' => $this->entity->renderAsIframe(),
    ];

    $form['arguments'] = [
      '#type' => 'plugins',
      '#title' => $this->t('Arguments'),
      '#default_value' => $this->entity->getArgumentConfig(),
      '#plugin_manager' => 'plugin.manager.spa_argument',
      '#plugin_name' => $this->t('Argument'),
    ];

    $form['libraries'] = [
      '#type' => 'plugins',
      '#title' => $this->t('Libraries'),
      '#default_value' => $this->entity->getLibraryConfig(),
      '#plugin_manager' => 'plugin.manager.spa_library',
      '#plugin_name' => $this->t('Library'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state)  {
    if ($form_state->getValue('iframe')) {
      $non_usable_arguments = [];

      /** @var SpaInterface $spa */
      $spa = $this->buildEntity($form, $form_state);
      foreach ($spa->getArguments() as $argument) {
        if (!$argument->passToIframe()) {
          $non_usable_arguments[] = $argument->getArgumentName();
        }
      }

      $this->messenger()->addWarning($this->t('The arguments @args won\'t be passed to the iframe.', [
        '@args' => implode(', ', $non_usable_arguments),
      ]));

    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $message = $result == SAVED_NEW
      ? $this->t('Created new SPA %label.', $message_args)
      : $this->t('Updated SPA %label.', $message_args);
    $this->messenger()->addStatus($message);
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    // Rebuild routes for new arguments.
    \Drupal::service('router.builder')->rebuild();
    // Clear library discovery for new libraries.
    \Drupal::service('library.discovery')->clearCachedDefinitions();
    // Clear block definitions for new/changed spa blocks.
    \Drupal::service('plugin.manager.block')->clearCachedDefinitions();
    return $result;
  }

}
