<?php

namespace Drupal\spa\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\spa\SpaInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Retrieves block plugin definitions for all spa blocks.
 */
class SpaBlockDeriver extends DeriverBase implements ContainerDeriverInterface {

  /**
   * The spa storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $spaStorage;

  /**
   * Constructs a SpaBlockDeriver object.
   *
   * @param \Drupal\Core\Entity\EntityStorageInterface $spa_storage
   *   The spa storage.
   */
  public function __construct(EntityStorageInterface $spa_storage) {
    $this->spaStorage = $spa_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    $entity_type_manager = $container->get('entity_type.manager');
    return new static(
      $entity_type_manager->getStorage('spa')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $spas = $this->spaStorage->loadByProperties(['status' => TRUE]);
    // Reset the discovered definitions.
    $this->derivatives = [];
    /** @var SpaInterface $spa */
    foreach ($spas as $spa) {
      $this->derivatives[$spa->id()] = $base_plugin_definition;
      $this->derivatives[$spa->id()]['context_definitions'] = $spa->getContextDefinitions();
      $this->derivatives[$spa->id()]['admin_label'] = $spa->label();
      $this->derivatives[$spa->id()]['config_dependencies']['config'] = [
        $spa->getConfigDependencyName(),
      ];
    }
    return parent::getDerivativeDefinitions($base_plugin_definition);
  }

}
