<?php

namespace Drupal\spa\Plugin\SpaArgument;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Plugin\Context\ContextDefinitionInterface;
use Drupal\spa\SpaArgumentPluginBase;

/**
 * Plugin implementation of the spa_argument.
 *
 * @SpaArgument(
 *   id = "context",
 *   label = @Translation("Context")
 * )
 */
class ContextSpaArgument extends SpaArgumentPluginBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + ['data_type' => '', 'required' => TRUE];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['data_type'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Context definition data type'),
      '#description' => $this->t('Examples: entity:user, wg_context:current_entity'),
      '#default_value' => $this->configuration['data_type'] ?? '',
    ];
    $form['required'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Context is required'),
      '#description' => $this->t('Non required contexts are experimental.'),
      '#default_value' => $this->configuration['required'] ?? TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getArgumentValue() {
    return $this->block->getContextValue($this->configuration['argument_name']);
  }

  /**
   * {@inheritdoc}
   */
  public function getContextDefinition(): ?ContextDefinitionInterface {
    $context_definition = ContextDefinition::create($this->configuration['data_type']);
    $context_definition->setLabel($this->configuration['argument_name']);
    $context_definition->setRequired($this->configuration['required']);

    return $context_definition;
  }

  /**
   * {@inheritdoc}
   */
  public function getDataType(): string {
    return $this->configuration['data_type'];
  }

  /**
   * {@inheritdoc}
   */
  public function passToIframe(): bool {
    // Currently we don't support to pass context arguments.
    return FALSE;
  }
}
