<?php

namespace Drupal\spa\Plugin\SpaArgument;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Utility\Token;
use Drupal\spa\SpaArgumentPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the spa_argument.
 *
 * @SpaArgument(
 *   id = "static",
 *   label = @Translation("Static")
 * )
 */
class StaticSpaArgument extends SpaArgumentPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The token service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Token $token) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->setConfiguration($configuration);
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition, $container->get('token'));
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'argument_description' => '',
      'default_value' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['argument_description'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Argument description'),
      '#description' => $this->t('Explain what this argument is doing for users who want to place this spa-block.'),
      '#default_value' => $this->configuration['argument_description'],
    ];
    $form['default_value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default value'),
      '#default_value' => $this->configuration['default_value'],
    ];
    $form['token_tree'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => [],
      '#dialog' => TRUE,
      '#weight' => 10,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm(array $form, FormStateInterface $form_state): array {
    $form = parent::blockForm($form, $form_state);
    $form[$this->configuration['uuid']] = [
      '#tree' => TRUE,
      'value' => [
        '#type' => 'textfield',
        '#title' => $this->t('The value for @name', ['@name' => $this->configuration['argument_name']]),
        '#description' => $this->t($this->configuration['argument_description']),
        '#default_value' => $this->getArgumentValue(),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getArgumentValue() {
    $value = $this->block->getConfiguration()['arguments'][$this->configuration['uuid']]['value'] ?? $this->configuration['default_value'];
    return $this->token->replace($value);
  }
}
