<?php

namespace Drupal\spa\Plugin\SpaLibrary;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\spa\SpaLibraryPluginBase;
use Symfony\Component\HttpFoundation\Response;

/**
 * Plugin implementation of the spa_library.
 *
 * @SpaLibrary(
 *   id = "css_snippet",
 *   label = @Translation("CSS Snippet")
 * )
 */
class CssSnippet extends SpaLibraryPluginBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['css'] = [
      '#type' => 'textarea',
      '#title' => $this->t('CSS'),
      '#default_value' => $this->configuration['css'],
      '#description' => $this->t('The CSS to include on the site'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'css' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getContent(): Response {
    return new Response(
      $this->configuration['css'],
      Response::HTTP_OK,
      ['content-type' => 'text/css']
    );
  }

  /**
   * {@inheritdoc}
   */
  public function setLibraryDefinition(&$definition): void {
    $library_url = Url::fromRoute('spa.library', [
      'spa' => $this->spa->id(),
      'library_uuid' => $this->configuration['uuid']
    ]);
    $definition['css']['component']['/' . $library_url->getInternalPath()] = [];
  }
}
