<?php

namespace Drupal\spa\Plugin\SpaLibrary;

use Drupal\Core\Form\FormStateInterface;
use Drupal\spa\SpaLibraryPluginBase;

abstract class ExternalBase extends SpaLibraryPluginBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['url'] = [
      '#type' => 'url',
      '#title' => $this->t('Script Url'),
      '#default_value' => $this->configuration['url'],
      '#description' => $this->t('The script url to include on the site.'),
    ];
    $form['minified'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Minified'),
      '#default_value' => !empty($this->configuration['minified']),
      '#description' => $this->t('Check if the script is minified.'),
    ];
    $form['async'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Async'),
      '#default_value' => !empty($this->configuration['async']),
      '#description' => $this->t('Check if the script shall be added with the async attribute.'),
      '#states' => [
        'visible' => [
          ':input[name*="[defer]"]' => ['checked' => FALSE],
        ],
      ]
    ];
    $form['defer'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Defer'),
      '#default_value' => !empty($this->configuration['defer']),
      '#description' => $this->t('Check if the script shall be added with the defer attribute.'),
      '#states' => [
        'visible' => [
          ':input[name*="[async]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'url' => '',
      'minified' => TRUE,
      'async' => FALSE,
      'defer' => FALSE,
    ];
  }
}
