<?php

namespace Drupal\spa\Plugin\SpaLibrary;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\multivalue_form_element\Element\MultiValue;
use Drupal\spa\SpaLibraryPluginBase;
use Symfony\Component\HttpFoundation\Response;

/**
 * Plugin implementation of the spa_library.
 *
 * @SpaLibrary(
 *   id = "js_snippet",
 *   label = @Translation("JS Snippet")
 * )
 */
class JsSnippet extends SpaLibraryPluginBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['js'] = [
      '#type' => 'textarea',
      '#title' => $this->t('JS'),
      '#default_value' => $this->configuration['js'],
      '#description' => $this->t('The JS to include on the site'),
    ];
    $form['dependencies'] += [
      'outside' => [
        '#type' => 'textfield',
        '#title' => $this->t('Outside variable name'),
      ],
      'inside' => [
        '#type' => 'textfield',
        '#title' => $this->t('Inside variable name'),
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'js' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getContent(): Response {
    $inside = array_column($this->getDependencies(), 'inside');
    $outside = array_column($this->getDependencies(), 'outside');
    $inside[] = 'drupalSettings';
    $outside[] = 'drupalSettings';
    $inside = implode(', ', $inside);
    $outside = implode(', ', $outside);
    return new Response(
      "(function (" . $inside . ") {\n" . $this->configuration['js'] . "\n})(" . $outside . ");",
      Response::HTTP_OK,
      ['content-type' => 'text/javascript']
    );
  }

  /**
   * {@inheritdoc}
   */
  public function setLibraryDefinition(&$definition): void {
    parent::setLibraryDefinition($definition);
    $library_url = Url::fromRoute('spa.library', [
      'spa' => $this->spa->id(),
      'library_uuid' => $this->configuration['uuid'],
    ]);
    $definition['js']['/' . $library_url->getInternalPath()] = ['minified' => TRUE];
  }
}
