<?php

namespace Drupal\spa\Plugin\media\Source;

use Drupal\block_field\Plugin\Field\FieldType\BlockFieldItem;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;

/**
 * @MediaSource(
 *   id = "block",
 *   label = @Translation("Block"),
 *   description = @Translation("Use a block as media."),
 *   allowed_field_types = {"block_field"},
 *   thumbnail_alt_metadata_attribute = "plugin_id",
 *   default_thumbnail_filename = "no-thumbnail.png"
 * )
 */
class BlockMediaSource extends MediaSourceBase {

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes() {
    return [
      'plugin_id' => $this->t('Block plugin ID'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadata(MediaInterface $media, $attribute_name)  {
    // Get the text_long field where the JSON object is stored
    $block_field = $media->get($this->configuration['source_field']);
    if ($block_field->first() instanceof BlockFieldItem) {
      $block = $block_field->first()->getBlock();
      if ($attribute_name == 'plugin_id') {
        return $block->getPluginId();
      }
    }

    return NULL;
  }

}
