<?php

namespace Drupal\spa\Routing;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\entity_instance\Entity\EntityInstanceConfigInterface;
use Drupal\entity_instance\EntityInstanceServiceInterface;
use Drupal\spa\SpaInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Route;

/**
 * Class EntityInstancesRoutes
 *
 * @package Drupal\entity_instances\Routing
 */
class SpaIframeRoutes implements ContainerInjectionInterface {


  protected $entityTypeManager;

  /**
   * EntityInstanceRoutes constructor.
   *
   * @param \Drupal\entity_instance\EntityInstanceServiceInterface $entity_type_manager
   *   The entity instance service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Route callback to register iframe routes.
   *
   * @return array
   *   The iframe routes
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function routes() {
    $routes = [];
    /** @var SpaInterface[] $spas */
    $spas = $this->entityTypeManager->getStorage('spa')->loadByProperties([
      'status' => 1,
      'iframe' => 1,
    ]);

    foreach ($spas as $spa) {
      $path = '/spa/iframe/' . $spa->id();
      $parameters = [];
      foreach($spa->getIframeArguments() as $argument) {
        $parameters[$argument->getArgumentName()] = ['type' => $argument->getDataType()];
        $path .= '/{' . $argument->getArgumentName() .'}';
      }

      $routes['spa.' .  $spa->id() . '.iframe'] = new Route(
        $path,
        [
          '_controller' => '\Drupal\spa\Controller\SpaController::iframe',
        ],
        [
          '_permission' => 'access content',
        ],
        [
          'spa' => $spa,
          'parameters' => $parameters,
        ]
      );
    }

    return $routes;
  }

}
