<?php

namespace Drupal\spa;

use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Context\ContextDefinitionInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\spa\Plugin\Block\SpaBlock;

/**
 * Interface for spa_argument plugins.
 */
interface SpaArgumentInterface extends PluginFormInterface, ConfigurableInterface {

  /**
   * Returns the translated plugin label.
   *
   * @return string
   *   The translated title.
   */
  public function label();

  /**
   * This is applied to the SPA block form.
   *
   * @param array $form
   * @param FormStateInterface $form_state
   * @return array
   */
  public function blockForm(array $form, FormStateInterface $form_state): array;

  /**
   * Get the block override value.
   *
   * @param FormStateInterface $form_state
   * @return array|null
   */
  public function getBlockFormValue(FormStateInterface $form_state): ?array;

  /**
   * Set the block.
   *
   * @param SpaBlock $block
   */
  public function setBlock(SpaBlock $block): void;

  /**
   * Setter for the spa config entity.
   *
   * @param SpaInterface $spa
   *   The SPA config entity.
   */
  public function setSpa(SpaInterface $spa): void;

  /**
   * @return string
   */
  public function getArgumentName(): string;

  /**
   * @return mixed
   */
  public function getArgumentValue();

  /**
   * @return mixed
   */
  public function getContextDefinition(): ?ContextDefinitionInterface;

  /**
   * Get the argument data type.
   *
   * @return string
   */
  public function getDataType(): string;

  /**
   * Defines whether the argument shall be passed to the iframe.
   *
   * @return bool
   *   True, if the argument shall be passed to the iframe.
   */
  public function passToIframe(): bool;

}
