<?php

namespace Drupal\spa;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Context\ContextDefinitionInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\spa\Plugin\Block\SpaBlock;

/**
 * Base class for spa_argument plugins.
 */
abstract class SpaArgumentPluginBase extends PluginBase implements SpaArgumentInterface {
  use StringTranslationTrait;

  /**
   * @var SpaBlock
   */
  protected $block = NULL;

  /**
   * The spa config entity
   *
   * @var SpaInterface
   */
  protected $spa;

  /**
   * {@inheritdoc}
   */
  public function setSpa(SpaInterface $spa): void {
    $this->spa = $spa;
  }

  /**
   * {@inheritdoc}
   */
  public function setBlock(SpaBlock $block): void {
    $this->block = $block;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['argument_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Argument name'),
      '#default_value' => $this->configuration['argument_name'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getArgumentName(): string {
    return $this->configuration['argument_name'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'argument_name' => 'arg',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = NestedArray::mergeDeep($this->defaultConfiguration(), $configuration);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    // Cast the label to a string since it is a TranslatableMarkup object.
    return (string) $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm(array $form, FormStateInterface $form_state): array {
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getBlockFormValue(FormStateInterface $form_state): ?array {
    return $form_state->getValue($this->configuration['uuid']);
  }

  /**
   * {@inheritdoc}
   */
  public function getContextDefinition(): ?ContextDefinitionInterface {
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getDataType(): string {
    return 'string';
  }

  /**
   * {@inheritdoc}
   */
  public function passToIframe(): bool {
    return TRUE;
  }

}
