<?php

namespace Drupal\spa;

use Drupal\Core\Config\Entity\ConfigEntityInterface;

/**
 * Provides an interface defining a spa entity type.
 */
interface SpaInterface extends ConfigEntityInterface {

  /**
   * Get the template.
   *
   * @return string
   */
  public function getTemplate(): string;

  /**
   * Get the SPA arguments.
   *
   * @return array
   */
  public function getArgumentConfig(): array;

  /**
   * Get the argument plugins.
   *
   * @return SpaArgumentInterface[]
   */
  public function getArguments(): array;

  /**
   * Get the argument plugin with given uuid.
   *
   * @return SpaArgumentInterface
   */
  public function getArgument(string $library_uuid): ?SpaArgumentInterface;

  /**
   * Get the spa libraries (CSS/JS).
   *
   * @return array
   */
  public function getLibraryConfig(): array;

  /**
   * Get the library name
   *
   * @return string
   *   The library name
   */
  public function libraryName(): string;

  /**
   * Get the library plugin instance for the specifies UUID.
   *
   * @param string $library_uuid
   *   The library UUID.
   * @return SpaLibraryInterface|NULL
   *   The SPA library.
   */
  public function getLibrary(string $library_uuid): ?SpaLibraryInterface;

  /**
   * Get the SPA libraries.
   *
   * @return SpaLibraryInterface[]
   */
  public function getLibraries(): array;

  /**
   * Get the library definition. for hook_library_info_build.
   *
   * @return array
   *   The library definition
   */
  public function libraryDefinition(): array;

  /**
   * Get the spa block config definitions.
   *
   * @return array
   */
  public function getContextDefinitions(): array;

  /**
   * @return bool
   *   True, if the template shall be rendered inside an iframe.
   */
  public function renderAsIframe(): bool;

  /**
   * @return SpaArgumentInterface[]
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getIframeArguments(): array;
}
