<?php

namespace Drupal\Tests\spa\FunctionalJavascript;

use Drupal\Core\Url;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * BigPipe regression tests.
 *
 * @group big_pipe
 */
class SpaFormTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'plugin_config_form_builder',
    'multivalue_form_element',
    'spa',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
  }

  public function testArgumentForm() {
    // @todo: Switch to non admin
    $this->drupalLogin($this->rootUser);
    $this->drupalGet(Url::fromRoute('entity.spa.add_form'));

    // Smoke test
    $this->assertSession()->pageTextContains('Add a SPA');

    // Basic settings.
    $page = $this->getSession()->getPage();
    $page->selectFieldOption('arguments[new_plugin][id]', 'static');
    $page->fillField('label', 'Test SPA');
    sleep(2);
    // $this->assertSession()->assertWaitOnAjaxRequest();

    // Add an argument
    $page->pressButton('Add Argument');
    sleep(1);
    $this->assertSession()->pageTextContains('Argument name');
    $page->pressButton('Add Argument');
    sleep(1);
    $uuids = $this->getUUIDs('arguments');
    $i = 0;
    foreach ($uuids as $uuid)  {
      $tab = $page->find('css', '[href^="#edit-arguments-'.$uuid.'"]');
      $tab->click();
      $page->fillField("arguments[$uuid][argument_name]", 'arg-' . $i);
      $page->fillField("arguments[$uuid][default_value]", 'value-' . $i++);
      sleep(1);
    }

    $page->find('css', '[value="Save"]')->click();
    $url = Url::fromRoute('entity.spa.collection');
    $this->assertSession()->addressEquals($url);

    // Check if there are 2 arguments in the config.
    $test_spa = \Drupal::config('spa.spa.test_spa');
    self::assertCount(2, $test_spa->get('arguments'));

    // Check if we can edit the spa
    $this->drupalGet(Url::fromRoute('entity.spa.edit_form', ['spa' => 'test_spa']));
    // Add a context more argument
    $page = $this->getSession()->getPage();
    $page->selectFieldOption('arguments[new_plugin][id]', 'context');
    $page->pressButton('Add Argument');
    sleep(1);

    self::assertCount(2, $uuids);
    // Remove the 0 arg
    $tab = $page->find('xpath', "//a[contains(@href, '#edit-arguments-".$uuids[0]."')]");
    $tab->click();
    $page->find('css', '[name="remove-plugin-'.$uuids[0].'"]')->click();
    sleep(1);
    $uuids = $this->getUUIDs('arguments');

    // Add a library
    $page->selectFieldOption('libraries[new_plugin][id]', 'css_snippet');
    $page->pressButton('Add Library');
    sleep(1);
    $library_uuids = $this->getUUIDs('libraries');
    $page->fillField('libraries['.$library_uuids[0].'][css]', '.foo { display: block; }');
    sleep(1);

    $page->find('css', '[value="Save"]')->click();
    $url = Url::fromRoute('entity.spa.collection');
    $this->assertSession()->addressEquals($url);

    // Check if there are 2 arguments in the config.
    $test_spa = \Drupal::config('spa.spa.test_spa');
    $arguments = $test_spa->get('arguments');
    self::assertCount(2, $arguments);
    self::assertEquals('static', $arguments[$uuids[0]]['id']);
    self::assertEquals('context', $arguments[$uuids[1]]['id']);
    self::assertEquals('.foo { display: block; }', $test_spa->get('libraries')[$library_uuids[0]]['css']);
  }

  protected function getUUIDs($type) {
    $uuids = [];
    $page = $this->getSession()->getPage();
    foreach ($page->findAll('css', '[data-drupal-selector="edit-'.$type.'-plugin-group"] input') as $element) {
      $name = $element->getAttribute('name');
      $matches = [];
      if (preg_match('/'.$type.'\[([^]]*)\]\[id\]/', $name, $matches)) {
        $uuids[] = $matches[1];
      }
    }

    return $uuids;
  }
}
